//  Copyright (c) 2006, David Corvoysier
//  Released under the GPL license
//  http://www.gnu.org/copyleft/gpl.html
//
// File: file.js
// Description: Kaizou file IO functions

// File input/output modes reminder
//PR_RDONLY     =0x01 lecture seulement
//PR_WRONLY     =0x02 criture seulement
//PR_RDWR       =0x04 lecture ou criture
//PR_CREATE_FILE=0x08 si le fichier n'existe pas, il est cr (sinon, sans effet)
//PR_APPEND     =0x10 le fichier est positionn  la fin avant chaque criture
//PR_TRUNCATE   =0x20 si le fichier existe, sa taille est rduite  zro
//PR_SYNC       =0x40 chaque criture attend que les donnes ou l'tat du fichier soit mis  jour
//PR_EXCL       =0x80 idem que PR_CREATE_FILE, sauf que si le fichier existe, NULL est retourne

// Delete a file
function deleteFile(file){
  try{
    file.remove(false);
  }
  catch(e)
  {
  	alert(e);
  }
}

// Delete a file corresponding to a path
function deleteFileAt(filepath){
  try{
    // Acquire the file interface
    var file =  Components.classes["@mozilla.org/file/local;1"]
                  .createInstance(Components.interfaces.nsILocalFile);
    // Init local file with file path (MUST be an absolute path)
    file.initWithPath(filepath);
    // Read the file
    deleteFile(file);
  }
  catch(e)
  {
    alert(e);
  }
}

function serializeXMLtoFile(file,dom){
  try{
    // Acquire the file output stream interface 
    var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                  .createInstance(Components.interfaces.nsIFileOutputStream);
    // Set flags PR_RDWR + PR_CREATE_FILE + PR_TRUNCATE
    var outputMode = 0x04 | 0x08 | 0x20;
    // Set rights
    var rights = 0664;
    // Init output stream
    outputStream.init(file, outputMode, rights, 0);
    // Serialize to stream
    var serializer = new XMLSerializer();
    serializer.serializeToStream(dom,outputStream,"UTF-8");
    // Close stream
    outputStream.close();
  }
  catch(e)
  {
  	alert(e);
  }
}

function writeFile(file,buffer){
  try{
    // Acquire the file output stream interface 
    var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                  .createInstance(Components.interfaces.nsIFileOutputStream);
    // Set flags PR_RDWR + PR_CREATE_FILE + PR_TRUNCATE
    var outputMode = 0x04 | 0x08 | 0x20;
    // Set rights
    var rights = 0664;
    // Init output stream
    outputStream.init(file, outputMode, rights, 0);
    // Write stream
    outputStream.write(buffer, buffer.length);
    // Close stream
    outputStream.close();
  }
  catch(e)
  {
  	alert(e);
  }
}

function writeFileAt(filepath,buffer){
  try{
    // Acquire the file interface
    var file =  Components.classes["@mozilla.org/file/local;1"]
                  .createInstance(Components.interfaces.nsILocalFile);
    // Init local file with file path (MUST be an absolute path)
    file.initWithPath(filepath);
    // Acquire the file output stream interface 
    writeFile(file,buffer);
  }
  catch(e)
  {
  	alert(e);
  }
}

// Read a file corresponding to an nsIFile Object
function readFile(file){
  try{
    // If file doesn't exist, return
    if ( file.exists() != true) {
      return;
    }
    // Acquire the file input stream interface 
    var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
           .createInstance( Components.interfaces.nsIFileInputStream );
    // Set flags PR_RDONLY
    var inputMode = 0x01;
    // Set rights
    var rights = 00004;
    // Init input stream
    inputStream.init(file, inputMode, rights, null);
    // Acquire the binary stream interface
    var sis = Components.classes["@mozilla.org/binaryinputstream;1"]
            .createInstance(Components.interfaces.nsIBinaryInputStream);
    // Bind it to opened stream
    sis.setInputStream(inputStream);
    // Read file content
    var output = sis.readBytes(sis.available());
    // Return file content
    return(output);
  }
  catch(e)
  {
    alert(e);
    return;
  }
}

// Read a file corresponding to a path
function readFileAt(filepath){
  try{
    // Acquire the file interface
    var file =  Components.classes["@mozilla.org/file/local;1"]
                  .createInstance(Components.interfaces.nsILocalFile);
    // Init local file with file path (MUST be an absolute path)
    file.initWithPath(filepath);
    // Read the file
    return readFile(file);
  }
  catch(e)
  {
    alert(e);
    return;
  }
}

// Load a Kaizou Description Document corresponding to an nsIFile Object
function loadKDDocument(file){
  // read file content
  var rawContent = readFile(file);
  if(rawContent){
    try{
      // Parse content
      var parser = new DOMParser();
      var doc = parser.parseFromString(rawContent,"text/xml");
      return doc;
    }
    catch(e)
    {
      alert(e);
      return;
    }
  }  
}

// Load a Kaizou Description Document corresponding to a path
function loadKDDocumentAt(path){
  // read file content
  var rawContent = readFileAt(path);
  if(rawContent){
    try{
      // Parse content
      var parser = new DOMParser();
      var doc = parser.parseFromString(rawContent,"text/xml");
      return doc;
    }
    catch(e)
    {
      alert(e);
      return;
    }
  }  
}

// Get the Kaizou scripts directory knowing the Kaizou path
function getKaizouDirectory(path){
  try{
      // Acquire the file interface
    var dir = Components.classes["@mozilla.org/file/local;1"].
              createInstance(Components.interfaces.nsILocalFile);
    // Go to the Kaizou directory
    dir.initWithPath(path);
    return dir;
  }
  catch(e){
    return null;
  }
}

// Load the Kaizou Description Documents contained in a directory corresponding to a path
function loadKDDocumentsAt(path){
  try{
    // Get the Kaizou active scripts directory
    var dir = getKaizouDirectory(path);
    if(dir && dir.exists()){
      // Load each file in the kaizou/active directory
      var docs = new Array();
      var dlst = dir.directoryEntries;
      while(dlst.hasMoreElements()) {
        var file = dlst.getNext().QueryInterface(Components.interfaces.nsIFile);
        if (file.isFile() && file.leafName.substring(file.leafName.lastIndexOf('.'))=='.xml') {
          // Load file
          var doc = loadKDDocument(file);
          if(doc){
            docs.push(doc);
          }
        }
      }
      return docs;
    }
    return;
  }
  catch(e)
  {
    alert(e);
    return;
  }
}

function saveKaizouDocument(file,doc){
  // Write document to disk
  serializeXMLtoFile(file,doc);
}

function saveKaizouDescriptionDocument(path,name,doc){
  try{
    // Get the Kaizou directory
    var dir = getKaizouDirectory(path);
    if(dir){
      if(!dir.exists()){
        // Create directory
        dir.create(1,777);
      }
      // Append name to the path
      dir.append(name);
      saveKaizouDocument(dir,doc);
    }
  }
  catch(e){
    alert(e);
  }
}

// Get the list of files in the Kaizoua directory
function getKaizouFiles(path){
  try{
    // Get the Kaizou active scripts directory
    var dir = getKaizouDirectory(path);
    if(dir && dir.exists()){
      // List each file in the kaizou/active directory
      var files = new Array();
      var dlst = dir.directoryEntries;
      while(dlst.hasMoreElements()) {
        var file = dlst.getNext().QueryInterface(Components.interfaces.nsIFile);
        if (file.isFile() && file.leafName.substring(file.leafName.lastIndexOf('.'))=='.xml') {
          // store file
          files.push(file);
        }
      }
      return files;
    }
    return;
  }
  catch(e)
  {
    alert(e);
    return;
  }
}
