//  Copyright (c) 2006, David Corvoysier
//  Released under the GPL license
//  http://www.gnu.org/copyleft/gpl.html
//
// File: manageDlg.js
// Description: manageDlg.xul js functions

// Helper functions
function getKaizouDescription(doc){
  var description = getFirstChildTextValue(doc,"description");
  return description ? description : "";
}

function getKaizouTitle(doc){
  var title = getFirstChildTextValue(doc,"title");
  return title ? title : "";
}

function getKaizouAuthor(doc){
  var author = getFirstChildTextValue(doc,"author");
  return author ? author : "";
}

function getListBox(){
  return document.getElementById("lstFiles");
}

function getSelectedItem(){
  return getListBox().selectedItem;
}

function getSelectedIndex(){
  return getListBox().getIndexOfItem(getSelectedItem());
}

function setDescription(value){
  document.getElementById("description").value = value;  
}

function setAuthor(value){
  document.getElementById("author").value = value;  
}

// Select the first item in the list
function selectFirstItem(){
  var listbox = getListBox();
  try{
    var first = listbox.getItemAtIndex(0);
  }catch(e){
    // There are no items in this list
    return;
  }
  listbox.selectItem(first);  
}

// Populate the files list box
function populateList(){
  var listbox = getListBox();
  // Get the files contained in the Kaizou directory
  var files = getKaizouFiles(getKaizouPath());
  for (var i = 0, file = null; (file = files[i]); i++) {
    var listitem = document.createElement("listitem");
    listitem.doc = loadKDDocument(file);
    listitem.file = file;
    listitem.setAttribute("label", getKaizouTitle(listitem.doc));
    listitem.setAttribute("crop", "end");
    listbox.appendChild(listitem);
  }
}

// Refresh the file description
function refreshDescription(){
  if(getSelectedItem()){
    var doc = getSelectedItem().doc;
    setDescription(getKaizouDescription(doc));
    setAuthor(getKaizouAuthor(doc));
  }else{
    setDescription("");
    setAuthor("");    
  }
}

// Uninstall a file
function doUninstall(){
  var doc = getSelectedItem().doc;
  var file = getSelectedItem().file;
  if(confirm("Do you want to permanently uninstall this Kaizou file ?")){
    // Delete file
    deleteFile(file);
    // Remove it from the list
    getListBox().removeItemAt(getSelectedIndex());
    // Select first item (if any)
    selectFirstItem();
    // Refresh the description box
    refreshDescription();
  }
}

// Init the dialog
function initDlg(){
  // Populate the file list
  populateList();
  // Select the first item (if any)
  selectFirstItem();
  // Refresh the description box
  refreshDescription();
}
